# cx52_tui.py : menu
import sys
import getopt
import string
import CX52

ALPHABET = string.ascii_uppercase
# ===================================
def isCapital(car):
    if string.ascii_uppercase.find(car) == -1:
        return 0
    else:
        return 1
# ===================================
def lst2str( aList ):
    return "".join( list( map( str, aList ) ))
# ===================================
def infoGlobal( info ):
    ch = ""
    ch += "Model: " + info["model"]
    ch += ", Configuration: " + info["config"]
    ch += "\nKdevice: " + str( info["kdevice"] )
    ch += "\nName of Wheels: " + ",".join( info["nameofwheels"])
    ch += "\n\nType Wheel\n=========="
    ch += "\nLeft  Alphabet : " + info["typewheel"]["alphaLeft"]
    ch += "\nRight Alphabet : " + info["typewheel"]["alphaRight"]
    ch += "\nCrown Alphabet : " + info["typewheel"]["crown"]
    if info["typewheel"]["annomaly"] != "":
        ch += "\n" + info["typewheel"]["annomaly"]
    ch += "\nSlide : " + str(info["typewheel"]["slide"])
    ch += "\nSpace  : " + info["typewheel"]["space"]
    ch += "\nMode  : " + info["typewheel"]["mode"]
    ch += "\nCounter: " + ("%03d" % (info["typewheel"]["counter"]))
    ch += "\n\nSquirrel Cage\n============="
    for i in range(38,-1,-1):
        if info["cage"]["thebars"][i] == None: continue
        num      = info["cage"]["thebars"][i]["num"]
        ident    = info["cage"]["thebars"][i]["ident"]
        theType  = info["cage"]["thebars"][i]["type"]
        lugs     = info["cage"]["thebars"][i]["lugs"]
        ch +=  "\n" + num + " " + ("%5s " %(ident)) + " "
        ch +=  ("%8s" % ("".join(theType))) + " "
        ch +=  "".join(list(map(str,lugs)))
    ch += "\n\nWheels\n======"
    for i in range(6):
        if info["wheels"][i] == None : continue
        ch += "\n\nWheel in position " + ("(%s) " % ("abcdef"[i]))
        ch += "\nName    : " + info["wheels"][i]["name"]
        ch += "\nLength  : " + str(info["wheels"][i]["length"])
        ch += "\nOffset  : " + str(info["wheels"][i]["offset"])
        ch += "\nShift   : " + str(info["wheels"][i]["shift"])
        ch += "\nRim     : " + info["wheels"][i]["rim"]
        ch += "\nNamepins: " + info["wheels"][i]["namepins"]
        ch += "\nPins    : " + info["wheels"][i]["pins"]
        ch += "\nComment : " + info["wheels"][i]["comment"]
    ch += "\n\n"
    return ch
# ===================================
def infoOnAChar( info ):
    counter = info["typewheel"]["counter"]
    slide   = info["typewheel"]["slide"]
    plain   = info["typewheel"]["plain"]
    crypto  = info["typewheel"]["crypto"]
    ch = "  "
    ch += " " + ("%03d" % (counter))
    ch += " " + plain
    ch += " " + crypto
    ch += " " + info["externalKey"]
    ch += " " + info["guides"]
    ch += " " + ("%2d" % (info["nbBarsShifted"]))
    ch += " " + ("%2d" % (info["shift"]))
    ch += " " + "A=" + ALPHABET[ int(slide) ]
    ch += " " + info["advances"]
    return ch
# ===================================
def getDefOpts( fileOpts, shortOpts, longOpts ):
    ch = list( map( str.strip, open( fileOpts, "r" ).readlines() ))
    opts, args = getopt.getopt( ch, shortOpts, longOpts )
    return opts
# ===================================
def usage():
    print("usage: python cx52_tui.py [option...] [default_opt_file]")
    print("The default option file contains default option in long format")
    print("  -h, --help")
    print("         Online Help")
    print("  -v, --verbose")
    print("         Verbose (mode Debug)")
    print("  -m, --model=model")
    print("         The CX model (FR, DE, ...), ZZ by default")
    print("  -c, --conf=config")
    print("         The configuration of the model, <<default>> by default")
    print("  -w, --wheels=wheels")
    print("         The six Key Wheels")
    print("         Ex: -w a29:a47:a43:a37:a41:a31")
    print("         You aren't obliged to set the six wheels")
    print("         Ex: -w a29:-:-:-:b29:-, by default:  -:-:-:-:-:-")
    print("  -p, --position=position")
    print("         The start position of the key wheels")
    print("         default is 01:01:01:01:01:01 or A:A:A:A:A:A (or mixed)")
    print("         If a wheel isn't here:  21:-01:-:-:-:B")
    print("  -a, --alpha=alphabet")
    print("         The alphabet, by default: ABC...Z")
    print("  -r, --reverse=reverse")
    print("         The reverse alphabet (on the right)")
    print("         by default the reverse of the previous alphabet")
    print("  -C, --crown=alphabet")
    print("         The crown where you choose the letter")
    print("         by default equal to alpha")
    print("  -S, --space=space")
    print("         The character for print space, Z by default")
    print("         It can be empty: -S ''")
    print("  -M, --mode=mode")
    print("         Mode: 'C' for cipher,'D' for decipher,'C' by default")
    print("  -s, --slide=slide")
    print("         The Slide, by default 0")
    print("  -f, --fv=fv")
    print("         The F/V mode, values: on/off, off by default")
    print("  -k, --kdevice=value")
    print("         The Kdevice (H54),from 1 to 8, 8 by default")
    print("  -t, --twobands")
    print("         Two bands flag, false by default")
# ------------------ Start of the Program  ---------------------
# ----- We manage options
DEBUG=False
MODEL="ZZ"
CONFIG="default"
NAMEOFWHEELS="-:-:-:-:-:-"
POSITION=""
ALPHA=""
REVERSE=""
CROWN=""
SPACE="Z"
MODE="C"
SLIDE=0
FV=False
KDEVICE=8
TWOBANDS=False
shortOpts = "hvm:c:w:p:a:r:C:S:M:s:f:k:t"
longOpts  = ["help", "verbose", 
        "model=", "conf=", "wheels=", "position=", "alpha=",
        "reverse=", "crown=", "space=", "mode=", "slide=",
        "fv=", "kdevice=", "twobands" ]
defOpts = ""
try:
    opts, args = getopt.getopt(sys.argv[1:], shortOpts, longOpts )
except getopt.GetoptError as err:
    usage()
    sys.exit(2)

if args != []:
    defOpts = getDefOpts( args[0], shortOpts, longOpts  )
    opts = defOpts + opts

for o, a in opts:
    if o == "-v":
        DEBUG = True
    elif o in ("-h", "--help"):
        usage()
        sys.exit()
    elif o in ("-m", "--model"):
        MODEL = a
    elif o in ("-c", "--conf"):
        CONFIG = a
    elif o in ("-w", "--wheels"):
        NAMEOFWHEELS = a
    elif o in ("-p", "--position"):
        POSITION = a
    elif o in ("-s", "--slide"):
        SLIDE=int(a)
    elif o in ("-k", "--kdevice"):
        KDEVICE=int(a)
    elif o in ("-f", "--fv"):
        if a.lower() == "on":
            FV = True
    elif o in ("-M", "--mode"):
        if a != 'C': MODE = 'D'
    elif o in ("-a", "--alpha"):
        ALPHA = a
    elif o in ("-r", "--reverse"):
        REVERSE = a
    elif o in ("-C", "--crown"):
        CROWN = a
    elif o in ("-S", "--space"):
        SPACE = a
    elif o in ("-t", "--twobands"):
        TWOBANDS = True
    else:
        assert False, "unhandled option"
#============================
leTexte = sys.stdin.readlines()
textIn = ""
textOut  = ""
textPlain = ""
for i in range(len(leTexte)):
    ch = leTexte[i]
    ch = ch.upper()
    for j in range(len(ch)):
        if not isCapital(ch[j]):
            continue
        car = ch[j]
        textIn += car
theCX = CX52.CX52( 
    theModel = {"name":MODEL,"conf":CONFIG},
    NAMEOFWHEELS = NAMEOFWHEELS,
    POSITION = POSITION,
    ALPHA = ALPHA,
    ALPHAREV = REVERSE,
    CROWN = CROWN,
    SPACE = SPACE,
    MODE = MODE,
    SLIDE = SLIDE,
    FV = FV,
    KDEVICE = KDEVICE,
)
if DEBUG:
    print("===> Default Options: ", defOpts )
    print( infoGlobal( theCX.information() ))
    print("Note: plain, cipher: in case we cipher, reverse if we decipher")
    print("(1): Counter, (2): Plain, (3): Cipher, (4) External key")
    print("(5): Active Pins, (6): Number of bars shifted")
    print("(7): Shift of the Type Wheel, (8): Slide")
    print("(9): Shift of key wheels")
    print("   1   2 3 4                  5      6  7  8    9 ")
    ch1 = theCX.getExtKey()
    ch2 = "A=" + ALPHABET[ theCX.typeWheel.slide ]
    cpt = theCX.typeWheel.counter
    print("  ", ("%03d" % (cpt)), "   ",ch1," "*12,ch2)

for char in textIn:
    charS, charIn, info = theCX.code( char )
    if DEBUG:
        print( infoOnAChar( info ) )
    textOut += charS
    textPlain += charIn
print(textOut)
if TWOBANDS:
    print( textPlain )
