# awheel.py
# obj: calculate active pin 

import sys
import string
import importlib.resources as pr
#==============================
class Wheel:
    def __init__(self,wheelname,thePins):
        #print(">>>", __package__ )
        info = pr.open_text( __package__ + ".data", wheelname + ".whl").readlines()
        self.name = wheelname
        self.position = 0
        self.lgth = int(info[0])
        self.offset = int(info[1])
        self.rim  = (info[2].rstrip()).split(":")
        self.nameOfPins = ((info[3]).rstrip()).split(":")
        self.shift = self.calculateShift()
        self.comment = info[4]
        self.pins = [0] * self.lgth
        for i in range( self.lgth ):
            self.pins[i] = int( thePins[i] )

    def calculateShift(self):
        if  self.nameOfPins[0].isnumeric():
            first = int( self.nameOfPins[0] )  
        else:
            first = 1
        return (self.offset + int(self.nameOfPins[0]) ) % self.lgth

    def information(self):
        info = {}
        info["name"] = self.name
        info["length"] = self.lgth
        info["offset"] = self.offset
        info["shift"] = self.shift
        info["rim"] =   ":".join(self.rim)
        info["namepins" ] = ":".join(self.nameOfPins)
        info["pins"] = "".join(list(map(str,self.pins)))    
        info["comment"] = self.comment
        info["externalkey"] = self.getExternalKey()
        return info

    def turnCounterClockWise(self, incr=1):
        self.position = (self.position + incr ) % self.lgth

    def turnClockWise(self, incr=1):
        self.position = (self.lgth + self.position - incr ) % self.lgth

    def getExternalKey(self):
        return self.rim[self.position]

    def setExternalKey(self,key):
        self.position = -1
        for i in range(self.lgth):
            if self.rim[i] == key:
                self.position = i
        if self.position == -1 : 
            #===> remplacer par une exception
            print("ERREUR cle externe", self.rim)
            sys.exit(1)

    def isActive(self):
        pos_active = ( self.position + self.shift - 1 ) % self.lgth
        if self.pins[ pos_active ]:
            return 1
        else:
            return 0

if __name__ == "__main__":
    print("====> test 'awheel.py'")
    x = Wheel("H29","00110011001100110110101100010")
    print( x.getExternalKey(), x.isActive() )
    x.setExternalKey("03")
    print( x.getExternalKey(), x.isActive() )
    x.turnCounterClockWise()
    print( x.getExternalKey(), x.isActive() )
    x = Wheel("D25","0000000000000010000000000")
    x.setExternalKey("A")
    print( "===>", x.shift, x.position )
    print( x.getExternalKey(), x.isActive() )

