# cage.py
# obj: manage the "squirrel cage"
#================================
import sys
import cxlibs.abar

class Cage:
    def __init__(self, filename ):
        fic = open(filename,"r")
        self.theBars = [None]*39
        while True:
            line=fic.readline()
            if line == "\n" : continue
            if line == "": break
            line=line.rstrip()
            num,ident,lugs = line.split()
            #print("Cage: ",num, ident, lugs)
            aBar = cxlibs.abar.Bar(num, ident, lugs)
            self.theBars[ int(num) ] = aBar 

    def information(self):
        info = {}
        info["thebars"] = [None]*39
        for i in range(32,0,-1): 
            if self.theBars[i] == None: continue
            info["thebars"][i] = self.theBars[i].information()
        return info

    def nbOfWheelSteps( self, guides ):
        advances = [0]*6
        for aBar in self.theBars:
            if aBar == None: continue
            advanceOfaBar = aBar.advanceOfWheels( guides )
            for j in range(6):
                advances[j] += advanceOfaBar[j]
        return advances

    def nbOfBarShifted( self, guides ):
        nbBars = {  
            "shifted":0, 
            "Tbars": 0,         
            "Kshifted":0,
            "Knoshifted":0,
            "Rshifted":0,
            "Rnoshifted":0,
        } 
        for aBar in self.theBars:
            if aBar == None: continue
            if aBar.isShifted( guides ):
                nbBars["shifted"] += 1
            typeOfBar = aBar.getTheType()
            if   typeOfBar == 'T':
                nbBars["Tbars"] += 1
            elif typeOfBar == 'K':
                if aBar.isShifted( guides ):
                    nbBars["Kshifted"] += 1
                else:
                    nbBars["Knoshifted"] += 1
            elif typeOfBar == 'R':   # Non-K
                if  aBar.isShifted( guides ):
                    nbBars["Rshifted"] += 1
                else:
                    nbBars["Rnoshifted"] += 1
        return nbBars

if __name__ == "__main__":
    print("====> test 'cage.py'")
    if ( len(sys.argv) < 2 ):
        print("Syntax: fichiers_de_barres")
    aCage = Cage( sys.argv[1] )
    print( aCage.information() )
    guides = [0,0,0,0,0,0]
    print( aCage.nbOfWheelSteps( guides ) )
    print( aCage.nbOfBarShifted( guides ) )

    guides = [1,1,0,1,0,1]
    print( aCage.nbOfWheelSteps( guides ) )
    print( aCage.nbOfBarShifted( guides ) )

