# typewheel.py
# 
import string

class TypeWheel:
    def __init__(self, 
            alphaLeft="", 
            alphaRight="", 
            crown="",
            space='Z', 
            slide=0, 
            mode='C',
            FV=False
        ):
        self.FV = FV
        self.space = space
        if alphaLeft == "":
            self.alphaLeft = string.ascii_uppercase
        else:
            self.alphaLeft = alphaLeft
        if alphaRight == "":
            ch = (self.alphaLeft[::-1])[-1]+ (self.alphaLeft[::-1])[:-1]
            self.alphaRight = ch
        else:
            self.alphaRight = alphaRight
        if crown == "":
            self.crown = self.alphaLeft
        else:
            self.crown = crown
        if self.alphaLeft != self.crown:
            self.annomaly = "WARNING! crown and Left alphabet are different"
        else:
            self.annomaly = ""
        self.lgth = len( self.alphaLeft )
        self.counter = 0
        self.slide = slide
        self.mode = mode

    def information(self):
        info = {}
        info["space"] = self.space
        info["alphaLeft"] = self.alphaLeft
        info["alphaRight"] = self.alphaRight
        info["crown"] = self.crown
        info["annomaly"] = self.annomaly
        info["slide"] = self.slide
        info["mode"] = self.mode
        info["counter"] = self.counter
        return info

    def cipher(self, plain, aShift ):
        info = {}
        self.counter += 1
        info["counter"] = self.counter
        pos1 = self.crown.find( plain )
        self.plain = self.alphaLeft[ pos1 ]
        info["plain"] = self.plain
        inputlet = self.plain
        pos2 = 52 + pos1 - aShift - self.slide
        output = self.alphaRight[ (pos2 % self.lgth) ] 
        info["crypto"] = output
        if self.mode == 'C' and ((self.counter % 5) == 0):
            output += ' '
            inputlet += ' '
        if self.mode == 'D' and output == self.space:
            output = ' '
        info["output"] = output
        info["input"] = self.plain
        if self.FV:
            self.advanceSlide( aShift )
        info["slide"] = self.slide
        return output, inputlet, info

    def advanceSlide(self, aShift ):
        self.slide = (self.slide + aShift) % 26

    def setSlide(self, slide ):
        self.slide = slide

    def getSlide(self ):
        return self.slide 

    def setFV(self, FV):
        self.FV = FV

    def getFV(self):
        return self.FV 

    def setMode(self, mode):
        self.mode = mode

    def getMode(self):
        return self.mode

    def resetCounter(self):
        self.counter = 0

    def getCounter(self):
        return self.counter

if __name__ == "__main__":
    print("=====> test 'typewheel.py'")
    tW = TypeWheel( "ABCDEFGHIJKLMNOPQRSTUVWXYZ" )
    print( tW.information() )
    tW.resetCounter()
    tW.setSlide(10)
    print( tW.cipher( "A",0 ) )
    print( tW.cipher( "B",0 ) )
    tW.advanceSlide( 5 )
    print( tW.cipher( "A",2 ) )
    print( tW.cipher( "B",3 ) )
    print( "Slide: ", tW.getSlide() )
    print( "%03d" % (tW.getCounter()))
    tW.setMode('D')
    print( "Mode: ", tW.getMode() )
    tW.setMode('C')
    print( "Mode: ", tW.getMode() )
    print( "FW: ", tW.getFV() )
    tW.setFV( True )
    print( "FW: ", tW.getFV() )

